/****************************************************************************************
* Copyright (c) 2010 Nathan Sala <sala.nathan@gmail.com>                               *
*                                                                                      *
* This program is free software; you can redistribute it and/or modify it under        *
* the terms of the GNU General Public License as published by the Free Software        *
* Foundation; either version 2 of the License, or (at your option) any later           *
* version.                                                                             *
*                                                                                      *
* This program is distributed in the hope that it will be useful, but WITHOUT ANY      *
* WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A      *
* PARTICULAR PURPOSE. See the GNU General Public License for more details.             *
*                                                                                      *
* You should have received a copy of the GNU General Public License along with         *
* this program.  If not, see <http://www.gnu.org/licenses/>.                           *
****************************************************************************************/

#include "TestDataEngine.h"
#include <QtTest>


TestDataEngine::TestDataEngine( QObject* parent ):QObject(parent)
    //: m_containment(Plasma::Containment()),
    //m_contextView(Context::ContextView(&m_containment, 0))
{
//     qDebug() << " coucou ";
//     m_contextWidget = new ContextWidget(0);
//     m_contextWidget->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
//     m_contextWidget->setSpacing( 0 );
//     m_contextWidget->setFrameShape( QFrame::NoFrame );
//     m_contextWidget->setFrameShadow( QFrame::Sunken );
//     m_contextWidget->setMinimumSize( 100, 100 );
//     qDebug() << " coucou 1\n";
//     m_contextScene = new Plasma::Corona(this);
//     qDebug() << " coucou 2\n";
//     m_containment = m_contextScene->createContainment(QString("testlol"));
//     qDebug() << " coucou 3\n";
//     m_contextView = new Context::ContextView(m_containment, m_contextScene, NULL);
}
