/***************************************************************************
 *   Copyright (c) 2024 Tuomas Nurmi <tuomas@norsumanageri.org>            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.         *
 ***************************************************************************/

#ifndef TESTPLAYLISTLAYOUT_H
#define TESTPLAYLISTLAYOUT_H

#include <QObject>
#include <QString>

class TestPlaylistLayout : public QObject
{
Q_OBJECT

public:
    TestPlaylistLayout();

private Q_SLOTS:
    void initTestCase();
    void testLoadingDefault();
    void testCreatingLayout();
    void testOrigins();
    void cleanupTestCase();

private:
    QString m_testPlaylistLayoutPath;
};


#endif // TESTPLAYLISTLAYOUT_H
